#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""
Some tools for Citizen Band users

Copyright (C) 2015 by
Sébastien Celles <s.celles@gmail.com>
All rights reserved.

"""

import numpy as np
import pandas as pd
import decimal

class CitizenBand(object):
    """
    A Python class for Citizen Band users

    >>> cb = CitizenBand()
    >>> cb.frequency(19)
    Decimal('27.185')

    >>> cb.frequency(19, 1)
    Decimal('27.635')

    >>> cb.frequency(19, -1)
    Decimal('26.735')

    >>> cb.channel('26.735')
    (19, -1, Decimal('0.000'))
    """

    def __init__(self, bands=None, mask=None):
        self.nb_channels = 40
        self.channels = pd.Index(np.arange(1, self.nb_channels + 1), name='channel')


        self.band_offset = decimal.Decimal('0.450')

        if bands is None:
            bands = [-2, -1, 0, 1, 2]
            bands = [-3, -2, -1, 0, 1, 2]
        self.bands = bands
        
        freq_bands = map(lambda band: self._get_frequencies_band(band), bands)
        self._df_freq_2d = pd.concat(freq_bands, axis=1)

        if mask is None:
            mask = self._df_freq_2d < decimal.Decimal('28')
    
        self._df_freq_2d = self._df_freq_2d[mask]

        freq_bands = map(lambda band: self._get_frequencies_band(band, with_band=True), bands)
        self._df_freq_1d = pd.concat(freq_bands, axis=0)
        self._df_freq_1d = self._df_freq_1d.reset_index()

        self._fmin = self._df_freq_1d['frequency'].min()
        self._fmax = self._df_freq_1d['frequency'].max()

    def _get_frequencies_band(self, band, with_band=False):
        _frequencies = pd.Series(
            ['26.965', '26.975', '26.985', '27.005', '27.015', '27.025', '27.035', '27.055', '27.065', '27.075', 
             '27.085', '27.105', '27.115', '27.125', '27.135', '27.155', '27.165', '27.175', '27.185', '27.205', 
             '27.215', '27.225', '27.255', '27.235', '27.245', '27.265', '27.275', '27.285', '27.295', '27.305', 
             '27.315', '27.325', '27.335', '27.345', '27.355', '27.365', '27.375', '27.385', '27.395', '27.405'],
            index=self.channels, name=band)
        _frequencies = _frequencies.map(decimal.Decimal) + self.band_offset * band
        if with_band:
            _frequencies.name = 'frequency'
            _frequencies = pd.DataFrame(_frequencies)
            _frequencies['band'] = band
        return _frequencies

    @property
    def frequencies(self):
        return self._df_freq_2d

    def frequency(self, channel, band=0):
        return self._df_freq_2d.loc[channel, band]

    def channel(self, frequency):
        try:
            frequency = decimal.Decimal(frequency)
        except:
            pass
        if frequency < self._fmin or frequency > self._fmax:
            raise NotImplementedError(
                "Out of bands %s - frequency should be between [%s, %s]" 
                % (self.bands, self._fmin, self._fmax))
        self._df_freq_1d['delta_f'] = self._df_freq_1d['frequency'] - frequency
        self._df_freq_1d['abs_delta_f'] = abs(self._df_freq_1d['delta_f'])
        self._df_freq_1d = self._df_freq_1d.sort_values(by='abs_delta_f')
        res = self._df_freq_1d.iloc[0]
        return res.channel, res.band, res.delta_f

def main():
    import doctest
    doctest.testmod()

    #cb = CitizenBand()

    #print("Table of frequencies")
    #print(cb.frequencies)

    #(channel, band) = (19, 0)
    #print("Frequency of channel %d band %d" % (channel, band))
    #frequency = cb.frequency(channel, band)
    #print(frequency)

    #frequency = decimal.Decimal('28.485')
    #print("Nearest channel and band of frequency %s" % (frequency))
    #channel, band = cb.channel(frequency)
    #print(channel, band)

if __name__ == '__main__':
    main()


__author__ = "Sebastien Celles"
__copyright__ = "Copyright 2015, celles.net"
__credits__ = ["Sebastien Celles"]
__license__ = "GPL"
__version__ = "0.0.1"
__maintainer__ = "Sebastien Celles"
__email__ = "s.celles@gmail.com"
__status__ = "Development"
__url__ = 'https://github.com/scls19fr/pyCitizenBand'



#!/usr/bin/env python
# -*- coding: utf-8 -*-

from setuptools import setup, find_packages  # Always prefer setuptools over distutils
from codecs import open  # To use a consistent encoding
from os import path
import io

here = path.abspath(path.dirname(__file__))

NAME = 'pyCitizenBand'
filename = path.join(here, NAME, "version.py")
with open(filename) as f:
    exec(f.read())

def readme():
    filename = path.join(here, 'README.rst')
    with io.open(filename, 'rt', encoding='UTF-8') as f:
        return f.read()

setup(
    name=NAME,

    # Versions should comply with PEP440.  For a discussion on single-sourcing
    # the version across setup.py and the project code, see
    # https://packaging.python.org/en/latest/development.html#single-sourcing-the-version
    #version='0.0.2',
    version=__version__,

    description='A Python package with some useful function for Citizen Band users, DXers, ...',
    long_description=readme(),

    # The project's main homepage.
    url=__url__,

    # Author details
    author=__author__,
    author_email=__email__,

    # Choose your license
    license=__license__,

    # See https://pypi.python.org/pypi?%3Aaction=list_classifiers
    classifiers=[
        # How mature is this project? Common values are
        #   3 - Alpha
        #   4 - Beta
        #   5 - Production/Stable
        'Development Status :: 3 - Alpha',

        # Indicate who your project is intended for
        'Environment :: Console',
        #'Topic :: Software Development :: Build Tools',
        'Intended Audience :: Developers',
        'Operating System :: OS Independent',

        # Specify the Python versions you support here. In particular, ensure
        # that you indicate whether you support Python 2, Python 3 or both.
        'Programming Language :: Cython',

        'Programming Language :: Python',
        #'Programming Language :: Python :: 2',
        #'Programming Language :: Python :: 2.6',
        'Programming Language :: Python :: 2.7',
        #'Programming Language :: Python :: 3',
        #'Programming Language :: Python :: 3.2',
        'Programming Language :: Python :: 3.3',
        'Programming Language :: Python :: 3.4',

        # Pick your license as you wish (should match "license" above)
        'License :: OSI Approved :: GNU General Public License v3 or later (GPLv3+)',

    ],

    # What does your project relate to?
    keywords='python pandas numpy radio ham radioham cb citizen band citizenband',

    # You can just specify the packages manually here if your project is
    # simple. Or you can use find_packages().
    packages=find_packages(exclude=['contrib', 'docs', 'tests*']),

    # List run-time dependencies here.  These will be installed by pip when your
    # project is installed. For an analysis of "install_requires" vs pip's
    # requirements files see:
    # https://packaging.python.org/en/latest/technical.html#install-requires-vs-requirements-files
    install_requires=['pandas'],

    # List additional groups of dependencies here (e.g. development dependencies).
    # You can install these using the following syntax, for example:
    # $ pip install -e .[dev,test]
    extras_require = {
        'dev': ['check-manifest', 'nose'],
        'test': ['coverage', 'nose'],
    },

    # If there are data files included in your packages that need to be
    # installed, specify them here.  If using Python 2.6 or less, then these
    # have to be included in MANIFEST.in as well.
    package_data={
        'samples': ['samples/*.py'],
    },

    # Although 'package_data' is the preferred approach, in some case you may
    # need to place data files outside of your packages.
    # see http://docs.python.org/3.4/distutils/setupscript.html#installing-additional-files
    # In this case, 'data_file' will be installed into '<sys.prefix>/my_data'
    #data_files=[('my_data', ['data/data_file'])],

    # To provide executable scripts, use entry points in preference to the
    # "scripts" keyword. Entry points provide cross-platform support and allow
    # pip to create the appropriate form of executable for the target platform.
    #entry_points={
    #    'console_scripts': [
    #        'sample=sample:main',
    #    ],
    #},
)

